# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2016-10-19 12:31
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Cache',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('ip', models.GenericIPAddressField(verbose_name='IP address')),
                ('permanent', models.BooleanField(default=False, verbose_name='Permanent')),
            ],
            options={
                'verbose_name_plural': 'Cache',
                'verbose_name': 'Cache entry',
            },
        ),
        migrations.CreateModel(
            name='Log',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('inserted', models.DateTimeField(auto_now_add=True, verbose_name='inserted')),
                ('message', models.CharField(max_length=1024, verbose_name='message')),
            ],
            options={
                'verbose_name_plural': 'Log',
                'ordering': ('-inserted',),
                'verbose_name': 'Log message',
            },
        ),
    ]



"""Abstraction layer to deal with Django related changes in order to keep
compatibility with several Django versions simultaneously."""
from __future__ import absolute_import, unicode_literals

from django.db import transaction


try:
    notrans = transaction.non_atomic_requests
except:
    notrans = transaction.commit_manually  # @UndefinedVariable


try:
    # Python 2
    import urllib2 as urllib
except ImportError:
    from urllib import request as urllib  # noqa @UnusedImport



from django.contrib import admin
from stopforumspam import models

class LogAdmin(admin.ModelAdmin):
    list_display = ('inserted', 'message')
    list_filter = ('inserted',)

class CacheAdmin(admin.ModelAdmin):
    list_display = ('ip', 'updated')
    list_filter = ('permanent', 'updated')


site = admin.site.register(models.Log, LogAdmin)
site = admin.site.register(models.Cache, CacheAdmin)



from django.conf import settings

CACHE_EXPIRE = getattr(settings, 'SFS_CACHE_EXPIRE', 1)  # days
LOG_EXPIRE = getattr(settings, 'SFS_LOG_EXPIRE', 30)  # days

# Inserted in Log model at every update
LOG_MESSAGE_UPDATE = getattr(
    settings, 'SFS_LOG_MESSAGE_UPDATE', u'Log updated from stopforumspam.com')

# Check all post requests
ALL_POST_REQUESTS = getattr(settings, 'SFS_ALL_POST_REQUESTS', False)

# Used if ALL_POST_REQUESTS=True
# Example: ["name_of_view", "/url/path/"]
URLS_IGNORE = getattr(settings, 'SFS_URLS_IGNORE', [])

# Used if ALL_POST_REQUESTS=False
# Example: ["name_of_view", "/url/path/"]
URLS_INCLUDE = getattr(settings, 'SFS_URLS_INCLUDE', [])

SOURCE_ZIP = getattr(settings, 'SFS_SOURCE_ZIP',
                     "http://www.stopforumspam.com/downloads/listed_ip_7.zip")
ZIP_FILENAME = getattr(settings, 'SFS_ZIP_FILENAME', "listed_ip_7.txt")

# Used for testing!
FORCE_ALL_REQUESTS = getattr(settings, 'SFS_FORCE_ALL_REQUESTS', False)

LOG_SPAM = getattr(settings, 'SFS_LOG_SPAM', True)

HEADER = getattr(settings, 'SFS_HTTP_HEADER', 'REMOTE_ADDR')
if HEADER != 'REMOTE_ADDR':
    HEADER = 'HTTP_' + HEADER.upper().replace('-', '_')



from __future__ import absolute_import, unicode_literals

import re
import zipfile
from io import BytesIO

from django.conf import settings
from django.core.management.base import BaseCommand
from django.db import transaction
from django.utils import timezone

from ... import settings as sfs_settings
from ... import compat, models


class Command(BaseCommand):
    help = 'Updates the database with the latest IPs from stopforumspam.com'

    def add_arguments(self, parser):
        parser.add_argument(
            '--force',
            action='store_true',
            dest='force',
            default=False,
            help='Force update of options',
        )
        parser.add_argument(
            '--quiet', '-q',
            action='store_true',
            dest='quiet',
            default=False,
            help='Do not produce output unless upon failure',
        )

    def handle(self, *args, **options):
        self.quiet = options['quiet']
        self.ensure_updated(force=options['force'])

    def print_output(self, msg):
        if not self.quiet:
            print(msg)

    def ensure_updated(self, force=False, quiet=False):

        last_update = models.Log.objects.filter(
            message=sfs_settings.LOG_MESSAGE_UPDATE).order_by('-inserted')
        do_update = force
        if not do_update and last_update.count() > 0:
            days_ago = timezone.now() - last_update[0].inserted
            if days_ago.days >= sfs_settings.CACHE_EXPIRE:
                do_update = True
        else:
            do_update = True
        if do_update:
            self.print_output("Updating (this may take some time)")
            self.print_output(
                "If you abort this command and want to rebuild, you have to use the --force option!")
            self.do_update()
        else:
            self.print_output("Nothing to update")

    @compat.notrans
    def do_update(self):
        try:
            self._do_update()
            transaction.commit()
        except Exception:
            transaction.rollback()
            raise

    def _do_update(self):
        # Delete old cache
        models.Cache.objects.filter(permanent=False).delete()

        # First log the update
        log = models.Log()
        log.message = sfs_settings.LOG_MESSAGE_UPDATE
        log.save()

        # For security purposes we test that each line is actually an IP
        # address
        ip_match = re.compile(r"^(\d+)\.(\d+)\.(\d+)\.(\d+)$")


        fileobject = None
        if sfs_settings.SOURCE_ZIP.startswith("file://"):
            filename = sfs_settings.SOURCE_ZIP.split("file://")[1]
            fileobject = open(filename, "rb")
        else:
            response = compat.urllib.urlopen(sfs_settings.SOURCE_ZIP)
            # Necessary because ZipFile needs to do a seek() call on the
            # file object which HttpResponse in python3 doesn't support.
            fileobject = BytesIO(response.read())
        z = zipfile.ZipFile(fileobject)
        ips = str(z.read(sfs_settings.ZIP_FILENAME))
        ips = ips.split("\n")
        ips = filter(lambda x: ip_match.match(x), ips)
        ips = list(ips)
        inserted = 0
        total = len(ips)
        objects_to_save = []
        for ip in ips:
            cache = models.Cache(ip=ip)
            objects_to_save.append(cache)
            inserted = inserted + 1
            if inserted % 100 == 0:
                self.print_output("Object %d of %d found" % (inserted, total))

        self.print_output("====================")
        self.print_output(" Saving to database ")
        self.print_output("====================")

        self.safe_bulk_create(objects_to_save)

        self.print_output("IPs from SFS saved to database")

    def safe_bulk_create(self, objs):
        """Wrapper to overcome the size limitation of standard bulk_create()"""
        if len(objs) == 0:
            return
        if 'sqlite' not in settings.DATABASES['default']['ENGINE']:
            objs[0].__class__.objects.bulk_create(objs)
        else:
            BULK_SIZE = 900 / len(objs[0].__class__._meta.fields)
            for i in range(0, len(objs), BULK_SIZE):
                self.print_output("SQLITE size limits: Bulk inserting %d objects" % BULK_SIZE)
                objs[0].__class__.objects.bulk_create(objs[i:i + BULK_SIZE])



from setuptools import find_packages, setup


from stopforumspam import __version__


CLASSIFIERS = [
    'Development Status :: 4 - Beta',
    'Environment :: Web Environment',
    'Framework :: Django',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: BSD License',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    'Topic :: Software Development',
    'Topic :: Software Development :: Libraries :: Application Frameworks',
]

packages = find_packages()

setup(
    name='stopforumspam',
    description='Django middleware for blocking IPs listed in stopforumspam.com',
    author="Benjamin Bach",
    author_email="benjamin@overtag.dk",
    version=__version__,
    packages=packages,
    license='BSD License',
    install_requires=[
        'Django>=1.7',
    ],
    url='https://overtag.dk/',
    classifiers=CLASSIFIERS,
    include_package_data=True,
)



from datetime import timedelta

from django.db import models
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _


from . import settings as sfs_settings



# Create your models here.
class Cache(models.Model):

    updated = models.DateTimeField(auto_now=True, verbose_name=_(u"Updated"))
    ip = models.GenericIPAddressField(verbose_name=_(u"IP address"),)
    permanent = models.BooleanField(
        default=False, verbose_name=_(u"Permanent"))

    class Meta:
        verbose_name = _(u"Cache entry")
        verbose_name_plural = _(u"Cache")


class Log(models.Model):
    inserted = models.DateTimeField(
        auto_now_add=True, verbose_name=_(u"inserted"))
    message = models.CharField(max_length=1024, verbose_name=_(u"message"))

    def save(self):
        super(Log, self).save()
        # Delete old log messages
        Log.objects.filter(
            inserted__lt=timezone.now() - timedelta(days=sfs_settings.LOG_EXPIRE)).delete()

    class Meta:
        ordering = ('-inserted',)
        verbose_name = _(u"Log message")
        verbose_name_plural = _(u"Log")
